package  
{
	import org.flixel.*;
	/**
	 * ...
	 * @author Simon Davies
	 */
	 
	public class Animal extends FlxParticle
	{
		// Heart particle
		[Embed(source="../assets/heart.png")] private var heartImage:Class;
		
		[Embed(source = "../assets/fish.png")] public static var fishImage:Class;
		[Embed(source = "../assets/lizard.png")] public static var lizardImage:Class;
		[Embed(source = "../assets/newt.png")] public static var newtImage:Class;
		[Embed(source = "../assets/sheep.png")] public static var sheepImage:Class;
		[Embed(source = "../assets/bird.png")] public static var birdImage:Class;
		[Embed(source = "../assets/dinosaur.png")] public static var dinosaurImage:Class;
		[Embed(source = "../assets/cloud.png")] public static var cloudImage :Class;
		
		// Sounds
		[Embed(source="../assets/sounds/evolve.mp3")] private var evolveSound:Class;
		
		public var species:String;
		private var startingSpecies:String;
		
		public function Animal(startingSpecies:String)
		{
			super();
			
			color = 0x88ffffff;
			
			this.startingSpecies = startingSpecies;
			setSpecies(startingSpecies);
			
			exists = false;
		}
		
		override public function reset(x:Number, y:Number):void {
			super.reset(x, y);
			setSpecies(startingSpecies);
		}
		
		public function setSpecies(newSpecies:String):void {
			species = newSpecies;
			if (newSpecies == "Fish") {
				loadGraphic(fishImage);
			}
			else if (newSpecies == "Newt") {
				loadGraphic(newtImage);
			}
			else if (newSpecies == "Lizard") {
				loadGraphic(lizardImage);
			}
			else if (newSpecies == "Sheep") {
				loadGraphic(sheepImage, false, true);
			}			
			else if (newSpecies == "Dinosaur") {
				loadGraphic(dinosaurImage);
			}			
			else if (newSpecies == "Bird") {
				loadGraphic(birdImage);
			}			
			else if (newSpecies == "Cloud") {
				loadGraphic(cloudImage);
			}
		}
		
		public function mutate(partner:Animal):void {
			
			var mutated:Boolean = false;
			
			if ((species == "Fish" && partner.species == "Newt") ||
			(species == "Newt" && partner.species == "Fish"))
			{
				setSpecies("Lizard");
				mutated = true;
			}			
			else if ((species == "Lizard" && partner.species == "Lizard") ||
			(species == "Lizard" && partner.species == "Lizard"))
			{
				setSpecies("Bird");
				mutated = true;
			}			
			else if ((species == "Lizard" && partner.species == "Cloud") ||
			(species == "Cloud" && partner.species == "Lizard"))
			{
				setSpecies("Sheep");
				mutated = true;
			}
			
			if (mutated) {
				FlxG.play(evolveSound);
				// Set new direction
				var new_x:Number = (velocity.x + partner.velocity.x) / 2;
				var new_y:Number = (velocity.y + partner.velocity.y) / 2;
				
				velocity.x = new_x;
				velocity.y = new_y;
				
				// you monster
				partner.kill();
				
				var emitter:FlxEmitter = new FlxEmitter(x, y,50);
				
				// Particles
				emitter.gravity = 0;
				emitter.maxRotation = 0;
				emitter.setXSpeed( -100, 100);
				emitter.setYSpeed( -100, 100);
				
				var particles:int = 50;
				// Create explosion effect
				for (var i:int = 0; i < particles; i++)
				{
					var particle:FlxParticle = new FlxParticle();
					particle.makeGraphic(1, 1);
					emitter.add(particle);
				}
				
				FlxG.state.add(emitter);
				emitter.start(true, 4,0,0);
			}
		}
		
		override public function update():void {
			super.update();
			angle = FlxU.getAngle(new FlxPoint(0, 0), velocity) + 90;
		
			// Check for blow streams
			var level:LevelBase = FlxG.state as LevelBase;
			var streams:FlxGroup = level.lifeStreams;
			
			var i:int;
			for(i = 0; i < streams.members.length; i++)
			{
				if (streams.members[i] is BlowStream) {
					// See if we can get blown around
					var blower:BlowStream = streams.members[i];
					
					var animalAngle:Number = FlxU.getAngle(new FlxPoint(x, y), new FlxPoint(blower.x, blower.y));
					
					if (Math.abs(blower.angle - animalAngle) > 170 && Math.abs(blower.angle - animalAngle) < 190) {
						
						var distance:Number = FlxU.getDistance(new FlxPoint(x, y), new FlxPoint(blower.x, blower.y));
						
						if (distance > 250) {
							continue;
						}
						else {				
							var xIncrease:Number = x - blower.x;
							var yIncrease:Number = y - blower.y;
							
							var magnitude:Number = Math.sqrt(Math.pow(xIncrease, 2) + Math.pow(yIncrease, 2));
							
							// Blow scalar
							var scalar:Number = 20 * (1.0 - (distance / 250));
							
							velocity.x += (xIncrease / magnitude * scalar);
							velocity.y += (yIncrease / magnitude * scalar);
						}
					}
				}
			}
		}
	}

}