package  
{
	import org.flixel.*;
	/**
	 * ...
	 * @author Simon Davies
	 */
	public class LevelBase extends FlxState
	{
		
		// Sounds
		[Embed(source = "../assets/sounds/kill.mp3")]private var killSound:Class;
		
		// Level files
		[Embed(source="../assets/level_0.csv", mimeType="application/octet-stream")] private var level_0_tiles:Class;
		
		// Tilemap
		[Embed(source = "../assets/tileset.png")] protected var tileMap:Class;
		
		[Embed(source="../assets/sounds/success.mp3")] private var completeSound:Class;
		
		public var lifeStreams:FlxGroup;
		public var endPoints:FlxGroup;
		
		protected var levelName:FlxText;
		
		protected var levelTilemap:FlxTilemap;
		
		public static var currentLevel:uint = 0;
		
		protected var progressText:FlxText;
		
		private var levelStart:Boolean = true;
		
		public function LevelBase() 
		{
			FlxG.mouse.show();
			
			levelName = new FlxText(FlxG.width - 128, 0, 128, "Level 1:\nHello world!");
			levelName.size = 14;
			
			progressText = new FlxText(0, 0, 128);
			progressText.size = 14;
			
			levelTilemap = new FlxTilemap();
			
			add(levelTilemap);		
			
			createLevel();
			
			add(progressText);
			add(levelName);
		}
		
		protected function loadNextLevel():void {
			currentLevel++;
			switch(currentLevel) {
				case 0:
					FlxG.switchState(new LevelBase());
					break;
				case 1:
					FlxG.switchState(new Level1());
					break;				
				case 2:
					FlxG.switchState(new Level2());
					break;				
				case 3:
					FlxG.switchState(new Level3());
					break;		
				case 4:
					FlxG.switchState(new Level4());
					break;				
				case 5:
					FlxG.switchState(new Level5());
					break;				
				case 6:
					FlxG.switchState(new Level6());
					break;				
				case 7:
					FlxG.switchState(new Level7());
					break;				
				case 8:
					FlxG.switchState(new Level8());
					break;				
				case 9:
					FlxG.switchState(new Level9());
					break;				
				case 10:
					FlxG.switchState(new Level10());
					break;				
				case 11:
					FlxG.switchState(new GameOver());
					break;
			}
		}
		
		public function createLevel():void {
			
			// Load map info
			levelTilemap.loadMap(new level_0_tiles, tileMap, 16, 16, FlxTilemap.OFF,0,1, 255);
			
			setupTileMap();
			
			lifeStreams = new FlxGroup(10);
			// New life stream
			lifeStreams.add(new LifeStream(470, 100, "Fish"));
			
			endPoints = new FlxGroup(10);
			endPoints.add(new LevelEnd(500, 300, "Fish"));
			
			// Add life streams to the game level
			add(lifeStreams);
			add(endPoints);
			
			// Help text
			var help1:FlxText = new FlxText(360, 70, 128, "This is a life stream. Click on it!");
			var help2:FlxText = new FlxText(400, 380, 128, "Direct the life stream to the life portal. " +
			"The symbol within the portal tells you which creature is needed to complete the level.");
			
			add(help1);
			add(help2);
		}
		
		protected function setupTileMap():void {
			levelTilemap.setTileProperties(1, FlxObject.NONE, inWater, Animal, 1);
			levelTilemap.setTileProperties(2, FlxObject.NONE, onSand, Animal, 12);
			levelTilemap.setTileProperties(15, FlxObject.NONE, onGrass, Animal, 12);
			levelTilemap.setTileProperties(28, FlxObject.NONE, onRock, Animal, 9);
		}
		
		override public function update():void {
			super.update();
			
			if (levelStart) {
				levelStart = false;			
				FlxG.flash(0xff000000, 2, null, true);
			}
			
			FlxG.overlap(lifeStreams, lifeStreams, animalCollisions);
			FlxG.overlap(lifeStreams, endPoints, checkForLevelCompletion);
			FlxG.collide(lifeStreams, levelTilemap);
					
			// Level end
			// Assume we have won unless we can prove otherwise
			var win:Boolean = true;
			var i:int;
			for(i = 0; i < endPoints.members.length; i++)
			{
				if (endPoints.members[i] is LevelEnd) {
					var end:LevelEnd = endPoints.members[i];
					if (!end.getCompleted()) {
						win = false;
						continue;
					}
				}
			}
			
			// Level end
			if (win) {
				FlxG.fade(0xff000000, 3, loadNextLevel, false);
			}
		}
		
		private function animalCollisions(object1:FlxObject, object2:FlxObject):void {
			
			// Animal collisions
			if (object1 is Animal && object2 is Animal && object1 != object2) {
				var animal:Animal = object1 as Animal;
				animal.mutate(object2 as Animal);
			}
		}
		
		private function checkForLevelCompletion(object1:FlxObject, object2:FlxObject):void {
			if (object1 is Animal && object2 is Portal) {
				var animal:Animal = object1 as Animal;
				var portal:Portal = object2 as Portal;
				
				if (animal.species == portal.parent.requirement) {
					FlxG.play(completeSound);
					animal.kill();
					FlxG.flash(0x66ffffff, 0.75, null, true);
					portal.parent.increment();
				}
			}
		}
		
		public function inWater(tile:FlxObject, object:FlxObject):void {
			if (object is Animal) {
				var animal:Animal = object as Animal;
				// Sheep can't swim?
				if (animal.species == "Sheep") {
					animal.kill();
					FlxG.play(killSound);
					FlxG.flash(0x55ff0000,1.0,null,true);
				}
			}
		}
		
		public function onSand(tile:FlxObject, object:FlxObject):void {
			if (object is Animal) {
				var animal:Animal = object as Animal;
				
				if (animal.species == "Fish" || animal.species == "Newt" || animal.species == "Sheep") {
					animal.kill();
					FlxG.play(killSound);
					FlxG.flash(0x55ff0000,1.0,null,true);
				}
			}
		}
		
		public function onGrass(tile:FlxObject, object:FlxObject):void {
			if (object is Animal) {
				var animal:Animal = object as Animal;
				
				if (animal.species == "Fish" || animal.species == "Newt") {
					animal.kill();
					FlxG.play(killSound);
					FlxG.flash(0x55ff0000,1.0,null,true);
				}
			}
		}		
		
		public function onRock(tile:FlxObject, object:FlxObject):void {
			if (object is Animal) {
				var animal:Animal = object as Animal;
				
				if (animal.species == "Fish" || animal.species == "Lizard" || animal.species == "Newt" || animal.species == "Sheep") {
					animal.kill();
					FlxG.play(killSound);
					FlxG.flash(0x55ff0000,1.0,null,true);
				}
			}
		}
		
	}

}