package  
{
	import org.flixel.*;
	/**
	 * ...
	 * @author Simon Davies
	 */
	public class LifeStream extends FlxGroup
	{
		
		private var _selected:Boolean;
		
		[Embed(source = "../assets/sounds/select.mp3")]private var selectSound:Class;
		[Embed(source="../assets/sounds/unselect.mp3")]private var unSelectSound:Class;
		
		public function set selected(value:Boolean):void {
			if (value == true) {
				FlxG.play(selectSound);
				selectionBox.visible = true;
				stream.selected = true;
				descText.exists = true;
			}
			else {
				if (_selected) {
					FlxG.play(unSelectSound);
				}
				selectionBox.visible = false;
				stream.selected = false;
				descText.exists = false;
			}
			_selected = value;
		}
		
		public function get selected():Boolean {
			return _selected;
		}
		
		private var enabled:Boolean;
		
		private var x:Number;
		private var y:Number;
		
		[Embed(source="../assets/selection.png")] private var selectImage:Class;
		
		private var selectionBox:FlxSprite;
		
		[Embed(source="../assets/spawner.png")] private var spawnerImage:Class;
		private var spawnerSprite:FlxSprite;
		
		[Embed(source="../assets/type_1.png")]  private var spawnerImageType:Class;
		private var spawnerTypeSprite:FlxSprite
		
		[Embed(source="../assets/line.png")] private var streamBeamImage:Class;
		[Embed(source="../assets/line2.png")] private var vectorLineImage:Class;
		
		private var stream:StreamEmitter;
		
		private var vector:FlxSprite;
		private var streamBeam:FlxSprite;
		
		private var species:String;
		
		private var angle:Number;
		
		private var descText:FlxText;
		
		public function LifeStream(x:Number, y:Number, species:String) 
		{
			super();
			
			this.x = x;
			this.y = y;
			
			this.species = species;
			
			descText = new FlxText(x + 64, y - 64, 128);
			
			angle = 90;
				
			streamBeam = new FlxSprite(x, y + 32 - 256,streamBeamImage);
			streamBeam.angle = angle;
			streamBeam.scale = new FlxPoint(1, 4);
			streamBeam.color = 0xffff00ff;
			add(streamBeam);
			
			vector = new FlxSprite(x + 28, y + 32 - 32,vectorLineImage);
			vector.angle = angle;
			vector.scale = new FlxPoint(1, 2);
			vector.color = 0xffffff00;
			add(vector);
			
			spawnerSprite = new FlxSprite(this.x, this.y);
			spawnerSprite.loadGraphic(spawnerImage);
			spawnerSprite.angularVelocity = 250;
			add(spawnerSprite);
			
			stream = new StreamEmitter(false, x + spawnerSprite.width / 2, y + spawnerSprite.height / 2, species);
			add(stream);
			
			selectionBox = new FlxSprite(x,y,selectImage);
			add(selectionBox);
			
			spawnerTypeSprite = new FlxSprite(x + spawnerSprite.width / 4 , y + + spawnerSprite.height / 4);
			spawnerTypeSprite.loadGraphic(spawnerImageType);
			
			if (species == "Fish") {
				spawnerSprite.color = 0xffff0000;
				descText.text = "Life Stream\n\nType: Fish\n\nCan Traverse: \nWater\n\nEvolution:\nNewt->Lizard"
			}
			else if (species == "Newt") {
				spawnerSprite.color = 0xff00ff00;
				descText.text = "Life Stream\n\nType: Newt\n\nCan Traverse: \nWater\n\nEvolution:\nFish->Lizard"
			}			
			else if (species == "Lizard") {
				spawnerSprite.color = 0xff00ffaa;
				descText.text = "Life Stream\n\nType: Lizard\n\nCan Traverse: \nWater\nSand\nGrass\n\nEvolution:\nLizard->Bird"
			}			
			else if (species == "Cloud") {
				spawnerSprite.color = 0xffaaaacc;
				descText.text = "Life Stream\n\nType: Cloud\n\nCan Traverse: All\n\nEvolution:\nLizard->Sheep"
			}			
			else if (species == "Sheep") {
				spawnerSprite.color = 0xffffffff;
				descText.text = "Life Stream\n\nType: Sheep\n\nCan Traverse: Grass\n\n"
			}
			
			add(spawnerTypeSprite);
			
			add(descText);
			descText.exists = false;
			
			selected = false;
		}
		
		override public function update():void {
			super.update();
			
			// Mouse selection
			if (FlxG.mouse.justPressed()) {
				if(!selected && FlxG.mouse.x > x && FlxG.mouse.y > y && FlxG.mouse.x < x + spawnerSprite.width && FlxG.mouse.y < y + spawnerSprite.height) {
					selected = true;
				}
				else {
					selected = false;
				}
			}
			
			// if selected, do mouse pointing stuff
			if (selected) {
				var mousePoint:FlxPoint = FlxG.mouse.getWorldPosition();
				var new_x_speed:Number = mousePoint.x - x;
				var new_y_speed:Number = mousePoint.y - y;
				
				var magnitude:Number = Math.sqrt(Math.pow(new_x_speed, 2) + Math.pow(new_y_speed, 2));
				
				new_x_speed /= magnitude;
				new_y_speed /= magnitude;
				
				new_x_speed *= StreamEmitter.speed;
				new_y_speed *= StreamEmitter.speed;
				
				angle = FlxU.getAngle(new FlxPoint(x, y), mousePoint);
				
				vector.angle = angle;
				streamBeam.angle = angle;
				
				stream.setXSpeed(new_x_speed, new_x_speed);
				stream.setYSpeed(new_y_speed, new_y_speed);
			}
		}
		
	}

}